//-< SerializableObjectFactory.java >--------------------------------*--------*
// GOODS                      Version 2.02       (c) 1998  GARRET    *     ?  *
// (Generic Object Oriented Database System)                         *   /\|  *
// Java Program Interface                                            *  /  \  *
//                          Created:     26-Sep-2002  K.A. Knizhnik  * / [] \ *
//                          Last update: 26-Oct-2002  K.A. Knizhnik  * GARRET *
//-------------------------------------------------------------------*--------*
// Factory for raw binary objects using Java serialization mechanism
//-------------------------------------------------------------------*--------*

package goodsjpi;

import java.io.*;

/**
 * Factory for raw binary objects using Java serialization mechanism
 */
public class SerializableObjectFactory implements RawBinaryFactory {
    public int size(Object obj) { 
        if (obj == null) { 
            return 0;
        }
	try { 
	    ByteArrayOutputStream out = new ByteArrayOutputStream();
	    ObjectOutputStream clu = new ObjectOutputStream(out);
	    clu.writeObject(obj);
	    clu.close();
            return out.size();
	} catch (IOException x) { 
	    throw new Error(x.getMessage());
	}
    }         

    static class ByteArrayPackStream extends OutputStream { 
        byte[] buf;
        int    pos;
        int    off;

        ByteArrayPackStream(byte[] arr, int offs) { 
            off = pos = offs;
            buf = arr;
        }
        
        public void write(byte b[], int off, int len) {
            System.arraycopy(b, off, buf, pos, len);
            pos += len;
        }

        public void write(int b) {
            buf[pos++] = (byte)b;
        }

        public int size() { 
            return pos - off;
        }

        public void close() {}
    }

    public int pack(Object obj, byte[] arr, int offs) { 
        if (obj != null) { 
            try { 
                ByteArrayPackStream out = new ByteArrayPackStream(arr, offs);
                ObjectOutputStream clu = new ObjectOutputStream(out);
                clu.writeObject(obj);
                clu.close();
                return out.size();
            } catch (IOException x) { 
                throw new Error(x.getMessage());
            }
        } 
        return 0;
    }

    public Object unpack(Class cls, byte[] arr, int off, int size) {
        if (size == 0) { 
            return null;
        }
	try { 
	    ByteArrayInputStream input = new ByteArrayInputStream(arr, off, size);
	    ObjectInputStream in = new ObjectInputStream(input);
	    return in.readObject();
	} catch (Exception x) { 
	    throw new Error(x.getMessage());
	}
    }

    public boolean supports(Class cls) { 
        return Serializable.class.isAssignableFrom(cls);
    }
}
